# <trigger>
# 	<usage>perl checkSiblingsandTransitionParent_noSecurity.pl parentReportName childReportName stateName attributeName validValues</usage>
# 	<version>4.4</version>
# 	<description>このトリガーは、すべての子 CR が特定の状態になったときに親 CR を自動的に遷移させるために使用します。
#                ある子 CR を使ってレポートを実行し、親 CR を取得します。
#                そして、別のレポートを実行して、この子 CR のすべての兄弟 CR を取得します。
#				 すべての兄弟が有効な状態にある場合は、親を遷移します。
#				 このトリガーは、ALM ライフサイクルのさまざまな箇所で使用されています。一例として、BCR の遷移可否のチェックがあります。
#                親レポート実行は Data_Report_RCP_BCR で、子レポート実行は Data_Report_BCR_RCP です。
#                子レポートを実行すると、すべての兄弟のリストが取得できます。
#                遷移先の状態名は bcr_in_progress であり、チェック対象の属性は crstatus です。
#				 有効な値は rcp_applied と rcp_rejected です。
#				 すべての RCP が正しい状態にある場合は、BCR は与えられた名称の状態へ遷移できます。
#			 	 チェック対象の属性は、パラメーターとして渡されるレポートのレポート定義に存在する必要があります。
#				 このトリガーは遷移の事後条件トリガーです。
#				 セキュリティー設定はなく、トリガーは与えられた遷移名を使わずに遷移名を生成します。
#				 ALM ライフサイクルにおいては、遷移はトリガーによって自動的に行われるため、「隠された遷移」となります。
#   </description>
# 	<argument1>
# 		<name>parentReportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>トリガー中に実行される親レポートの名称。</description>
#   </argument1>
# 	<argument2>
# 		<name>childReportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>トリガー中に実行される子レポートの名称。</description>
#   </argument2>
# 	<argument3>
# 		<name>stateName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>遷移先の状態名。</description>
#   </argument3>
# 	<argument4>
# 		<name>attributeName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>チェック対象の属性の名称。</description>
#   </argument4>
# 	<argument5>
# 		<name>validValues</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>属性をチェックする際に有効と見なされる値。個々の値はセミコロンで区切ります。
#                    属性は指定された値の 1 つと一致する必要があります。一致しないと CR は遷移されません。
#       </description>
#   </argument5>
#  </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $parentreport = shift;
my $childreport = shift;
my $stateName = shift;
my $attrToCheck = shift;
my $attrValueList = shift;

my @validAttrValues = split /;/, $attrValueList;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";

my $data = $csapi->QueryData($user, $parentreport, $queryString, undef, undef, undef);

#Get the child data, this is the associated data, the sub-report data.
my $childData = $data->getDataObject(0)->getChildData();

#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
for(my $i = 0; $i < $childData->getDataSize(); $i++)
{
	#Get the object vector
	my $objVec = $childData->getDataObject($i);

	# Run a query to get the child data of the parent.  This is to find all the siblings of the initial object
	my $problem_number = $objVec->getDataObjectByName("problem_number")->getValue();
	my $subqueryString = "problem_number='" . $problem_number . "'";

	my $subdata = $csapi->QueryData($user, $childreport, $subqueryString, undef, undef, undef);
	
	# Now that we have the parent object, we find all of its children. These are the siblings.
	my $sibling = $subdata->getDataObject(0)->getChildData();
	for (my $j=0; $j < $sibling->getDataSize(); $j++)
	{
		my $siblingData = $sibling->getDataObject($j);
		my $match = 0;
		# Check to make sure the attribute to check has one of the valid values, if it doesn't, the script exits
		# without performing the transition.
		foreach my $value (@validAttrValues)
		{
			if ($siblingData->getDataObjectByName($attrToCheck)->getValue() eq "\L$value")
			{
				$match = 1;
			}
		}
		if ($match == 0)
		{
			exit(0);
		}
	}

	#The name of the transition is the current state of the object, the string "2", and the state to transition to
	my $transitionName = "HIDDEN_" . $objVec->getDataObjectByName("crstatus")->getValue() . "2" . $stateName;

	#Get the transition show form.
	my $tmp = $csapi->TransitionCRData($user, $problem_number , $transitionName);	

	print "Automatically transitioned CR $problem_number to $stateName\n";

	#Set the crstatus to the state we want the CR to end up in.
	$tmp->getDataObjectByName("crstatus")->setValue($stateName);

	#Do the transition
	$csapi->TransitionCR($user, $tmp);	
}
