#  <trigger>
# 	<usage>perl createChild.pl relationSubmitName</usage>
# 	<version>4.4</version>
# 	<description>CR が正しく遷移したときに、与えられたコピー提出フォームを使用して子 CR を自動的に作成します。
#      継承されるデータとして設定された任意の情報は正しく提出されます。このトリガーは通常事後遷移トリガーとして実行されます。
#   </description>
# 	<argument1>
# 		<name>relationSubmitName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>子 CR を作成する際に使用するコピー提出フォーム名。</description>
#        </argument1>
#  </trigger>

use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

if(@ARGV < 2)
{
	print "Must supply two parameters: relation name and the location of the trigger file.";	
	exit 1;
}

my $relation    = shift;
my $triggerFile = shift;

#Create a new TriggerParser object
my $trigger = new ChangeSynergy::TriggerParser($triggerFile);

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();
my $parentCR   = $trigger->get_object_id();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(),
					$trigger->get_token(), $trigger->get_database());

#Do a copy
my $submitForm = $csapi->CopyCRData($user, $parentCR, $relation);

#For each objectData on the submit form.
for(my $i = 0; $i < $submitForm->getDataSize; $i++)
{
	#Is the object required?
	if($submitForm->getDataObject($i)->getRequired())
	{
		#see if the object is already set with a value.
		if(length($submitForm->getDataObject($i)->getValue()) == 0)
		{
			$submitForm->getDataObject($i)->setValue("Not Set");
		}	
	}
	
	#If the value is inherited make sure that its modify value is set
	#to true so that it will get submitted. 
	if($submitForm->getDataObject($i)->getInherited())
	{
		$submitForm->getDataObject($i)->setIsModified("true");
	}
}

#set the crstatus to the correct state.
$submitForm->getDataObjectByName("crstatus")->setValue($submitForm->getTransitionLink(0)->getToState());	

#Do the submit of the CR and associate it to its parent CR
my $tmpStr = $csapi->SubmitCRAssocCR($user, $submitForm, $submitForm->getTransitionLink(0)->getRelation(), $parentCR);

