#  <trigger>
# 	<usage>perl transitionParent.pl reportName stateName</usage>
# 	<version>4.4</version>
# 	<description>子 CR が正しく遷移できた場合に、レポートを実行して現在の CR に関連付けられた親 CR を取得します。
#                    その後、「状態名」に指定された状態への親 CR の遷移を試みます。
#                    トリガーは遷移が利用可能かどうかをチェックし、可能であれば親 CR は遷移されます。
#   </description>
# 	<argument1>
# 		<name>reportName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>親 CR を検索するために実行するレポートの名称。</description>
#   </argument1>
# 	<argument2>
# 		<name>stateName</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>親 CR の遷移先である状態。</description>
#   </argument2>
#  </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

#Get the name of the report to run and the end state for the CRs to be transitioned to.
my $report = shift;
my $stateName = shift;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $
				      trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";
my $data = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);

#Empty array which will hold the problem numbers for the problems to be transitioned.
my @problemsToTransition = ();

#For each top level csapi_cobject_vector_postion.
for(my $i = 0; $i < $data->getDataSize(); $i++)
{
	#Get the child data, this is the associated data, the sub-report data.
	my $childData = $data->getDataObject($i)->getChildData();

	#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
	for(my $j = 0; $j < $childData->getDataSize(); $j++)
	{
		#Get the object vector
		my $objVec = $childData->getDataObject($j);

		push (@problemsToTransition, $objVec->getDataObjectByName("problem_number")->getValue());
	}
}

my $size = @problemsToTransition;

#For each problem in the problemsToTransition array.
foreach my $problem (@problemsToTransition)
{
	#do an AttributeModifyCRData show action.
	my $theproblem = $csapi->AttributeModifyCRData($user, $problem, "crstatus");

	#For each allowed transition.
	for(my $i = 0; $i < $theproblem->getTransitionLinkSize(); $i++)
	{
		#Get the transition object
		my $transLink = $theproblem->getTransitionLink($i);
		
		#See if the transition is the one we want. 
		my $value     = $transLink->getToState() cmp "\L$stateName";

		if($value == 0)
		{
			#Get the transition show form.
			my $tmp = $csapi->TransitionCRData($user, $problem, $transLink->getTransition());
			
			#Set the crstatus to the state we want the CR to end up in.
			$tmp->getDataObjectByName("crstatus")->setValue($transLink->getToState());

			#Do the transition
			$csapi->TransitionCR($user, $tmp);	
		}
	}
}
