# <trigger>
#	<usage>perl createRelation.pl relationName attributeName relationType</usage>
#	<version>4.3</version>
#	<description>attributeName パラメーターで指定された CR から、このスクリプトを起動した CR に対して関係を作成します。
#		attributeName パラメーターには、RCR オブジェクトの CR 番号を含む遷移の属性の名前が含まれます。   
#	</description>
#	<argument1>
#		<name>relation</name>
#		<required>true</required>
#		<type></type>
#		<description>CR 間に作成する関係の名前</description>
#	</argument1>
#	<argument2>
#		<name>attributeName</name>
#		<required>true</required>
#		<type></type>
#		<description>CR 番号を含む属性の名前。</description>
#   </argument2>
#	<argument3>
#		<name>rel_type</name>
#		<required>true</required>
#		<type></type>
#		<description>作成する関係のタイプ。使用できる値は、problem_problem、problem_task、problem_object、task_task、task_object、および object_object です。
#		</description>
#	</argument3>
# </trigger>
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $relation = shift;
my $relation_attr = shift;
my $rel_type = shift;

#Create a new TriggerParser object
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

my %attributes = $trigger->get_attributes();
my $create = $attributes{$relation_attr};

if((!defined($create)) || ("\L$create" eq ""))
{
	exit 0;
}

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();
my $fromObject   = $trigger->get_object_id();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(),
					$trigger->get_token(), $trigger->get_database());

#verify that the target object exists.
#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $create . "'";
my $data = $csapi->QueryData($user, "Basic Summary", $queryString, undef, undef, undef);

if ($data->getDataSize() == 0) {
	die "Unable to create relation, CR " . $create . " does not exist\n";
}

my $tmpstr = $csapi->CreateRelation($user, "FALSE", $create, $fromObject, $relation, $rel_type);
print "Created $relation relationship between $fromObject and $create\n";
exit 0;
