# <trigger>subscriptionTrigger.pl<usage></usage>
#	<version>4.4</version>
#	<description>发送电子邮件至一个用户列表，这些用户当前预订了已修改的 CR。
# 仅当订户不是 CR 的修改者时，才会接收到一封电子邮件。
#                注：在使用该脚本前必须先编辑 pt.cfg 文件。有两个必须填写的配置项：SMTP_SERVER 和 FROM_EMAIL_ADDRESS。
#                     SMTP_SERVER 项用于定义 SMTP 服务器的地址，发送电子邮件时应使用该服务器。FROM_EMAIL_ADDRESS 项将用作为通过触发器发送的所有电子邮件中的“发件人”。
#   </description>
# </trigger>

use utf8;
use ChangeSynergy::emailHelper;
use strict;                         #Make sure all variables are defined before use.
use ChangeSynergy::TriggerParser;   #Used to parse the trigger XML file sent from ChangeSynergy.
use ChangeSynergy::csapi;           #Main module for the CS API.

#Create a new TriggerParser object, passing in the location of the trigger XML document.
#The trigger XML document is always the last parameter sent to a trigger.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Will die if smtp server or from address not set.
ChangeSynergy::EmailHelper::verifyMailData($trigger);

#Get all the attributes from the trigger file.
my %trigger_attributes = $trigger->get_attributes();

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object, password is not needed, uses the token
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "", $trigger->get_role(),
		   $trigger->get_token(), $trigger->get_database());

#Setup the EmailHelper module so it can share the csapi, user and trigger variables.
ChangeSynergy::EmailHelper::setup($csapi, $user, $trigger);

#Get the crid from the trigger.
my $crid = $trigger->get_object_id();

#Get extra CR information from the server
my $crInfo = ChangeSynergy::EmailHelper::getCrDetails($crid, "problem_synopsis|crstatus");

#Get the subscriber email addresses, this list is comma delimited since Mail::Sender takes a comma
#delimited list.
my $subscribers = $trigger->get_subscriber_email_addresses();

#Use the ChangeSynergy::EmailHelper to open the email and get things going.
ChangeSynergy::EmailHelper::openEmail("", "", $subscribers, "IBM Rational Change subscription notification - CR " . $crid);

my $mainTableData = ChangeSynergy::EmailHelper::createCrInfoBlockData($crInfo, "Modifier");
ChangeSynergy::EmailHelper::sendStrippedTable("Change Request Information", $mainTableData);

my $additionalTableData = createAdditionalBlockData();
ChangeSynergy::EmailHelper::sendStrippedTable("Modified Attributes", $additionalTableData);

#Closes the email out and attaches the inline image.
ChangeSynergy::EmailHelper::closeEmail();

######################## Local Methods ########################

sub createAdditionalBlockData
{
	my $crInfo = shift;

	my @loop_data = ();  # initialize an array to hold the loop data.
	my $i = 1;

	my @keys = keys %trigger_attributes;

	#Get a hash of attribute aliases as the keys and attriubte names as the values.
	my %keyAliasesNames = ChangeSynergy::EmailHelper::getAttributeLabels(@keys);

	#create a | separated list of attribute names that can be used in getCrDetails method.
	my $attributeNames = "";
	foreach my $key (sort(keys %keyAliasesNames))
	{
		my $attrName = $keyAliasesNames{$key};
		
		$attributeNames = ($attributeNames eq "" ? $attributeNames : $attributeNames."|").$attrName;
	}
	
	my $crData = ChangeSynergy::EmailHelper::getCrDetails($crid, $attributeNames);
	
	#For each of the attributes that were changed print out the change in the email.
	foreach my $key (sort(keys %keyAliasesNames))
	{
		my $attrName = $keyAliasesNames{$key};

		if (($attrName ne "problem_number") && ($attrName ne "modify_time") && ($attrName ne "COMPVER_ARR"))
		{
			my %row_data = ();
			$row_data{alias} = $key;
			
			if ($attrName ne "_COMMENTS" && $crData->getDataObjectByName($attrName)->getType() eq "CCM_DATE") 
			{
				my $dateValue = $trigger_attributes{$attrName};				
				if (length($dateValue) == 13) 
				{
					$dateValue = $dateValue / 1000;
				}
				my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($dateValue);
				my $y = 1900 + $year;
				my $m = 1 + $mon;				
				my @dateArray = ($y,$m,$mday,$hour,$min,$sec);
				
				foreach (@dateArray)
				{	
					if (length($_) == 1)
					{
						$_ = "0$_";
					}				
				}
				
				#date format yyyy/MM/dd hh:mm:ss
				my $date = "$dateArray[0]/$dateArray[1]/$dateArray[2] $dateArray[3]:$dateArray[4]:$dateArray[5]";				 
				$row_data{value} = ChangeSynergy::util::htmlEncode($date);
			}
			else 
			{
				if ($crData->getDataObjectByName($attrName)->getType() eq "CCM_EXTENDED_TEXT")
				{
					$row_data{value} = $trigger_attributes{$attrName};
				}
				else
				{
					$row_data{value} = ChangeSynergy::util::htmlEncode($trigger_attributes{$attrName});
				}
			}
			
			$row_data{zebra} = $i % 2;
			push(@loop_data, \%row_data);
			$i++;
		}
	}

	return \@loop_data; 
}
