// Copyright IBM Corporation 2000, 2009

/*
<trigger>
	<usage>bsf create_associated_task.js "See CR %problem_number: %problem_synopsis"</usage>
	<version>5.0</version>
	<description>
		在提交或转变期间将新任务与 CR 关联。仅当名为 _CREATE_TASK 的属性作为转变请求的一部分发送时，才会创建此任务。通常，通过在转变表单中包含类型为 CCM_TOGGLE 的 _CREATE_TASK 的属性来执行此操作，这样，用户只需单击复选框即可决定是否在转变期间创建新任务。
	</description>
	<argument1>
		<name>synopsis</name>
		<required>no</required>
		<type>string</type>
		<description>
			要提供给新任务的大纲。包含空格的大纲应使用双引号括起。通过在属性名称前加百分号 (%)，大纲可引用父 CR 中的属性。例如，缺省任务大纲为“查看 CR %problem_number: %problem_synopsis”，它包含父 problem_number 和 problem_synopsis。如果引用不存在的属性，那么该属性的名称会在大纲中按原样保留。
		</description>
	</argument1>
	<argument2>
		<name>-releaseattr cr_release_attribute_name</name>
		<required>no</required>
		<type>string</type>
		<description>
			通常，会将“release”属性从 CR 复制到新任务。如果指定，那么允许您使用 CR 中的另一个属性作为任务发行版，例如，-releaseattr target_release。
		</description>
	</argument2>
	<argument3>
		<name>-resolverattr cr_user_attribute</name>
		<required>no</required>
		<type>string</type>
		<description>
			通常，会将“resolver”属性从 CR 复制到新任务。如果指定，那么允许您使用 CR 中的另一个属性作为任务解决者，例如，-resolverattr enterer。
		</description>
	</argument3>
</trigger>
*/

importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

// check if the _CREATE_TASK pseudo-attribute is present.
// If so, we should create a new task.
var createTask = false;
for (var i = 0; i < event.attributes.length; i++)
{
	if (event.attributes[i].name == '_CREATE_TASK')
	{
		createTask = true;
		break;
	}
}

if (createTask)
{
	// Some lifecycles may want to set the task 'resolver' and 'release' from
	// custom attributes on the CR. Check for custom attribute names, or use the default if there are none.
	var options = OptionParser.parse(args);
	var resolverAttrName = options.getOption('-resolverattr') || 'resolver';
	var releaseAttrName = options.getOption('-releaseattr') || 'release';

	// use the given task synopsis (a template), or a default
	var synopsisTemplate = new KeywordString(options.args[0] || 'See CR %problem_number: %problem_synopsis');

	// get all the attribute values we need from the CR that we'll apply to the task.
	// This includes the attributes needed to expanded the attribute keywords in the synopsis.
	var crAttributes = change.collectAttributes(
		event.triggerUser.token,
		event,
		[resolverAttrName, releaseAttrName].concat(synopsisTemplate.getKeywords()));

	var taskSynopsis = synopsisTemplate.replaceKeywords(mapToStringValues(crAttributes));

	var taskAttributes = [
		new Attribute('task_synopsis', taskSynopsis),
		new Attribute('priority', 'medium')
		];
		
	// ensure the inherited attributes are on the CR before we use them.
	// They may not be required for this transition
	var crResolver = crAttributes.get(resolverAttrName);
	var crRelease = crAttributes.get(releaseAttrName);
	if (crResolver && crResolver.stringValue) taskAttributes.push(new Attribute('resolver', crResolver.stringValue));
	if (crRelease && crRelease.stringValue) taskAttributes.push(new Attribute('release', crRelease.stringValue));

	var taskId = change.createAssociatedTask(
		event.triggerUser.token,
		event.objectId,
		taskAttributes);

	log.debug('CR ' + event.objectId + ': associated new task ' + taskId);
}

/**
 * Changes Map<String, Attribute> into a Map<String, String>,
 * using the Attribute string values as the new map's values.
 * The latter form can be passed to replaceKeywords.
 */
function mapToStringValues(attributeMap)
{
	var valueMap = new java.util.HashMap();
	for (var iter = attributeMap.keySet().iterator(); iter.hasNext();)
	{
		var key = iter.next();
		valueMap.put(key, attributeMap.get(key).toString());
	}

	return valueMap;
}
