// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf create_relationship.js relationship -from|-to other_cr_attribute</usage>
	<version>5.0</version>
	<description>
		在转变期间，在两个现有 CR 之间创建关系。其中一个 CR 是要转变的 CR；另一个 CR 的编号在转变期间作为属性提交。用于指定“另一个”CR 编号的属性名称是该脚本的自变量。
		关系的方向由 -from 或 -to 自变量确定。这些选项引用“另一个”CR，而非要转变的 CR。例如，要创建从 _DUPLICATE 属性中所提及的 CR 到要转变的 CR 的“重复”关系，您应使用自变量“duplicate -from _DUPLICATE”调用此脚本。
	</description>
	<argument1>
		<name>relationship</name>
		<required>yes</required>
		<type>string</type>
		<description>要创建的关系的名称。</description>
	</argument1>
	<argument2>
		<name>-from|-to other_cr_attribute</name>
		<required>yes</required>
		<type>string</type>
		<description>发送至转变的属性的名称，该属性包含与要转变的 CR 相关的其他 CR 的 CR 号。例如，缺省的“标记为重复”转变上使用的 _DUPLICATE 属性用于指定将当前 CR 标记为哪个 CR 的重复项。使用 -to 将创建到该 CR 的关系；-from 将创建源自该 CR 的关系。
			请注意，以下划线开头的属性名称不会保存到 CR，因此，这样的名称就是用于传输诸如此类瞬态信息的一种不错选择。  
		</description>
	</argument2>
</trigger>
*/
importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

var options = OptionParser.parse(args);

if (options.args.length != 1
	|| (options.getOption('-to') == null && options.getOption('-from') == null))
{
	// check for three args, and that the second is one of the known option values
	usage();
}
else
{
	var relationship = options.args[0];
	var reverseDirection = options.getOption('-from');
	var attributeWithOtherCrid = options.getOption('-to') || options.getOption('-from');
	
	if (options.getOption('-to'))
	{
		var fromCrid = event.objectId;
		var toCrid = findOtherCrid(options.getOption('-to'));
	}
	else
	{
		var fromCrid = findOtherCrid(options.getOption('-from'));
		var toCrid = event.objectId;

	}
	
	if (toCrid && fromCrid)
	{
		log.debug('Creating ' + relationship + ' from ' + fromCrid + ' to ' + toCrid);

		try
		{
			change.relateNoSecurity(
				event.triggerUser.token,
				fromCrid,
				toCrid,
				relationship);
		}
		catch (e)
		{
			if (e.javaException)
			{
				event.fail(e.javaException.getMessage());
			}
			else
			{
				throw e;
			}
		}
	}
}

/**
 * Finds the named attribute on the event and returns it string value.
 * This is assumed to be the CRID of the "other" CR involved in this relationship.
 * @param inAttribute name of a modifed attribute that contains the CR ID to relate with.
 * @return the value of the named attribute, or undefined if not found.
 */
function findOtherCrid(inAttribute)
{
	for (var i = 0; i < event.attributes.length; i++)
	{
		// use .equals, since these are Java strings; == would compare their references, not their values.
		if (event.attributes[i].name.equals(inAttribute))
		{
			return event.attributes[i].stringValue;
		}
	}
}

function usage()
{
	log.error('Could not create relationship: missing arguments.');
	log.error('Usage: create_relationship.js relationship -from|-to attribute_containing_crid_to_relate_to');
	log.error('Example: create_relationship.js duplicate -from _DUPLICATE');
}