/*
<trigger>
	<usage>bsf set_CRPR_phase_attribute.js [-ifnotset true]</usage>
	<version>5.0</version>
	<description>
		设置关联任务属性 CRPR_phase 的值。</description>
	<argument1>
		<name>-ifnotset true/false</name>
		<required>no</required>
		<type>boolean</type>
		<description>
			如果为 true，那么当目标对象已具有该属性的任何值时，不会设置该属性。这用于一次性设置值，如果同一个触发器可以多次运行，那么不会再次更新这些值。如果未指定，会缺省设置为 false（即使属性具有值，也会设置该属性）。
		</description>
	</argument1>
</trigger>
*/


// Import packages
importPackage(Packages.com.telelogic.cs.api);

// Script specific global variables
const script_name = 'set_CRPR_phase_attribute.js';
const script_version = '20080620';
const script_usage = 'set_CRPR_phase_attribute.js [-ifnotset true]';
const script_example = 'set_CRPR_phase_attribute.js';
const log_header = 'Script bsf ' + script_name + ': ';
const script_debug = false;
const script_dcm_modifiable_check = true;

// Get objects
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');
var options = OptionParser.parse(args);

if (script_debug) 
{
	log.info(log_header);
	log.info('    Version: ' + script_version);
	log.info('    Arguments: ' + args.join(' '));
	log.info('    Current CR: ' + event.fromObjectId);
}

if (checkArguments(0, ['-ifnotset']))
{
	// Get Options
	var to = 'CRPR_phase';

	var statetoPhaseMap =
	{
		"in_analysis":"analysis",
		"in_resolution":"resolution",
		"in_evaluation":"evaluation"						
	}

	var crStatus = change.getAttributes(event.triggerUser.token, event.fromObjectId, ['crstatus']);
	crStatus = crStatus.get('crstatus');

	var value = statetoPhaseMap[crStatus];
	
	var if_not_set = options.getOption('-ifnotset');

	// Create attribute object for value
	var attribute = createAttribute(to, value);

	if (script_debug) 
	{
		if (isAttributeNull(attribute))
		{
			log.info(log_header + 'Value is null');
		}
		else
		{
			log.info(log_header + 'Value is \'' + attribute + '\' and is of type \'' + attribute.type + '\'');
		}
	}

	// Branch over relationship type
	var state_attribute_name;
	var attribute_names = [to];
	var items;
	
	// Associated tasks
	if (script_debug) 
	{
		log.info(log_header + 'Working on associated tasks');
	}
	
	attribute_names.push('from_db_path');
	
	if (script_dcm_modifiable_check)
	{
		attribute_names.push('modifiable_in');
	}

	items = change.getAssociatedTasks(event.triggerUser.token, event.fromObjectId, attribute_names);
	
	// Filter list
	if (script_debug)
	{
		log.info(log_header + 'Item list: ' + items.join(', '));
	}

	items = filterItems(items, attribute, state_attribute_name, if_not_set);
	
	if (script_debug) 
	{
		log.info(log_header + 'Filtered item list: ' + items.join(', '));
	}
	
	// Action item
	for (var i = 0; i < items.length; i++)
	{
		setObjectAttribute(items[i], attribute);
	}
}


/**
 * 
 * Checks the script arguments:
 * - Check argument count
 * - Check that all specified options are valid
 * - Check that all required options are specified
 * 
 * Returns: true, if succesfull. false otherwise
 */
function checkArguments(arg_count, allowed_options)
{
	var message;

	// Check argument count
	if (options.args.length < arg_count)
	{
		message = 'Insufficient arguments to run the script (' + options.args.length +' arguments)';
	}

	// Check that all specified options are valid
	if (!message)
	{
		var all_options = options.getOptions();
		var i = all_options.keySet().iterator();
	
		while (!message && i.hasNext())
		{
			var option = i.next();
			var is_valid = false;
		
			for (var j = 0; j < allowed_options.length; j++)
			{
				if (option == allowed_options[j])
				{
					is_valid = true;
					break;
				}
			}
			
			if (!is_valid)
				message = 'Invalid option \'' + option + '\'';
		}
	}
	
	if (message)
	{
		log.error(log_header);
		log.error('   ' + message);
		log.error('   Usage: ' + script_usage);
		log.error('   Example: ' + script_example);
		return false;
	}

	return true;
}

/**
 * Checks, if an attribute has no value set.
 * 
 * The function is necessary, because Attribute.toString() returns 'null' instead of null. By
 * returning 'null' it cannot be distinguished between an without a value and one with the 
 *  value 'null'.
 * 
 * Returns: True, if the attriute has no value set.
 */ 
function isAttributeNull(attribute)
{
	if (((!attribute) || (attribute.type == Attribute.Type.STRING) && (attribute.getStringValue() == null))
		|| ((attribute.type == Attribute.Type.DATE) && (attribute.getDateValue() == null))
		|| ((attribute.type == Attribute.Type.BOOLEAN) && (attribute.getBooleanValue() == null)))
	{
		return true;
	}
	
	return false;
}


/**
 * 
 * Creates an attribute object for the attribute of the specified name
 * 
 * The function is necessary to ensure that the attribute object has the correct type.
 * If created with the wrong type the attribute is cleared.
 * 
 * Returns: New Attribute object
 */
function createAttribute(name, value)
{
	if (!name)
	{
		return null;
	}

	if (!value)
	{
		return new Attribute(name);
	}

	// Lookup attribute type in the lifecycle
	var attributedefs = change.getAttributeDefs(event.triggerUser.token);
	var type;
	
	for (var i = 0; i < attributedefs.length; i++)
	{
		if (attributedefs[i].name.equals(name)) // == is always false
		{
			type = attributedefs[i].type;
			break;
		}
	}

	if (!type)
	{
		// Do not write an error. It could be an attribute for a task
		if (script_debug) 
		{
			log.info(log_header + 'Attribute ' + name + ' has no attribute definition in the lifecycle. Using string instead');
		}

		return new Attribute(name, value.toString());
	}

	// The type will be the one from the lifecycle editor not Attribute.Type
	if (type == 'time')
	{
		value = new Date(value);
	}
	else if (type == 'boolean')
	{
		value = new Boolean(value);
	}
	else
	{
		value = value.toString();
	}

	return new Attribute(name, value);
}

/**
 * Filters items from the array of items to be handled by the script
 * - The to attribute has already the desired value
 * - The state is the state to be excluded
 * - The item is not modifiable in the current database and script_dcm_modifiable_check is true
 * 
 * Returns: Filtered list of items
 */
function filterItems(items, attribute, state_attribute_name, ifNotSet)
{
	var new_items = new Array();
	
	for (var i = 0; i < items.length; i++)
	{
		// The to attribute has already the desired value
		// Check explicitely for empty attributes (see isAttributeNull for details) 
		if ((isAttributeNull(items[i].getAttribute(attribute.getName())) && isAttributeNull(attribute)) || 
			(!isAttributeNull(items[i].getAttribute(attribute.getName())) && !isAttributeNull(attribute) &&
			 items[i].getAttribute(attribute.getName()).toString().equals(attribute.toString())) // == is always false
			)
		{
			if (script_debug) 
			{
				log.info(log_header + 'Skipping equal value \'' + attribute.toString() + '\' in item ' + items[i]);
			}
			
			continue;
		}

		if (ifNotSet)
		{
			// Don't reset an attribute is it's already set to some value.
			if (items[i].getAttribute(attribute.getName()))
			{
				if (script_debug)
				{
					log.info(log_header + 'Skipping already set value \'' + attribute.toString() + '\' in item ' + items[i]);
				}

				continue;
			}
		}
		
		// The item is not modifiable in the current database and script_dcm_modifiable_check is true
		if (script_dcm_modifiable_check &&
			change.isDcmEnabled(event.triggerUser.token) &&
			!items[i].getAttribute('modifiable_in').toString().equals(change.getDbid(event.triggerUser.token)) &&
			!(change.isCentralServer(event.triggerUser.token) && items[i] instanceof Task)) // == is always false
		{
			if (script_debug) 
			{
				log.info(log_header + 'Skipping non modifiable item. Item \'' + items [i]+ 
				'\' is not modifiable in database \'' + change.getDbid(event.triggerUser.token) + '\'');
			}
			
			continue;
		}

		new_items.push(items[i]);
	}

	return new_items;
}


/*
 * 
 * Sets an attribute on a CR or task
 *
 */
function setObjectAttribute(item, attribute)
{
	var is_task = (item.toString().indexOf('Task') == 0);
	var attributes = [attribute];
	var item_id_string;

	try 
	{
		if (is_task)
		{
			// Set attribute on a task
			item_id_string = 'Task ' +  item.taskId;
			if (script_debug) 
			{
				log.info(log_header + 'Modifying ' + item_id_string);
			}
	
			// Override the context database in central server mode
			var from_db_path = item.getAttribute('from_db_path');
			if (from_db_path)
			{
				attributes.push[from_db_path];
			}
	
			change.setTaskAttributes(event.triggerUser.token, item.taskId, attributes);	
		}
		else 
		{
			// Set attribute on a CR
			item_id_string = 'CR ' +  item.crid;
			if (script_debug) 
			{
				log.info(log_header + 'Modifying ' + item_id_string);
			}
	
			change.setAttributes(event.triggerUser.token, item.crid, attributes);	
		}
	}
	catch (error)
	{
		var message = change.getMessage('errorSettingAttributeXonItemYerrorZ', [attribute.name, item_id_string, error.message || error.toString()]);
		log.error(log_header + 'Copying from CR ' + event.fromObjectId + '. ' + message); 
		event.fail(message);
	}
}
